import { NextFunction, Request, Response } from "express";
import prisma from "../database/dbclient";
import bcrypt from "bcryptjs";
import { z } from "zod";
import { CompanySchema } from "../types/types.schema";
/// <reference path="../types/types.schema.ts"/>

// Create a new company
export const createCompany = async (req: Request, res: Response) => {
  const {
    company_name,
    industry,
    headquarters,
    contact_number,
    founded_year,
    website,
    logo,
    is_current_user_owner,
    is_active,
  } = req.body;

  try {
    const newCompany = await prisma.companies.create({
      data: {
        company_name,
        industry,
        headquarters,
        contact_number,
        founded_year,
        website,
        logo,
        is_active,
      },
    });

    if (is_current_user_owner != null && is_current_user_owner == true) {
      /// assgining current user as a admin of this company

      const user_id = req.user_id;

      const company = await prisma.usercompany.create({
        data: {
          user_id: user_id,
          company_id: newCompany.company_id,
          company_role_id: 3, // 3 means company admin
        },
      });

      // now assgin this user as a employee to the role table

      const user_roles = await prisma.user_roles.create({
        data: {
          user_id: user_id,
          role_id: 3, // 3 means company admin
        },
      });
    }

    res.status(201).json(newCompany);
  } catch (error) {
    console.error(
      "POST > Create a Company > Failed to create a company",
      error
    );
    res.status(500).json({ message: "Error creating company", error });
  }
};

// Get all companies

// export const getCompanies = async (req: Request, res: Response) => {
//     try {
//         const companies = await prisma.companies.findMany();
//         res.status(200).json(companies);
//     } catch (error) {
//         res.status(500).json({ message: 'Error fetching companies', error });
//     }

// };

// get all  company
export const getCompanies = async (req: Request, res: Response) => {
  const { page = 1, limit = 25 } = req.query;

  const pageNum = parseInt(page as string);
  const limitNum = parseInt(limit as string);

  const skip = (pageNum - 1) * limitNum;

  try {
    const users = await prisma.companies.findMany({
      take: limitNum,
      skip: skip,
    });

    const total = await prisma.companies.count();
    const total_page = Math.ceil(total / limitNum);

    res.status(200).json({
      total_page,
      data_count: users.length,
      next_page: pageNum < total_page ? pageNum + 1 : undefined,
      previous_page: pageNum > 1 ? pageNum - 1 : undefined,
      data: users,
    });
  } catch (error) {
    res.status(500).json({ message: "Error fetching companies", error });
  }
};

// Get a single company by ID
export const getCompanyById = async (req: Request, res: Response) => {
  const { id } = req.params;

  try {
    const company = await prisma.companies.findUnique({
      where: { company_id: parseInt(id) },
    });

    if (!company) {
      return res.status(404).json({ message: "Company not found" });
    }

    res.status(200).json(company);
  } catch (error) {
    console.error(
      "GET > Get company by id > Failed to get company details\n",
      error
    );
    res.status(500).json({ message: "Error fetching company", error });
  }
};

// Update a company by ID
export const updateCompany = async (req: Request, res: Response) => {
  const { id } = req.params;
  const {
    company_name,
    industry,
    headquarters,
    contact_number,
    founded_year,
    website,
    logo,
    is_active,
    email,
  } = req.body;

  try {
    const updatedCompany = await prisma.companies.update({
      where: { company_id: parseInt(id) },
      data: {
        company_name,
        industry,
        headquarters,
        contact_number,
        founded_year,
        website,
        logo,
        is_active,
        email,
      },
    });

    res.status(200).json(updatedCompany);
  } catch (error) {
    res.status(500).json({ message: "Error updating company", error });
  }
};

// Delete a company by ID
export const deleteCompany = async (req: Request, res: Response) => {
  const { id } = req.params;

  try {
    await prisma.companies.delete({
      where: { company_id: parseInt(id) },
    });

    res.status(204).send();
  } catch (error) {
    res.status(500).json({ message: "Error deleting company", error });
  }
};

// Assign a new User to a Company
export const assignUserCompany = async (req: Request, res: Response) => {
  const { company_id, user_id, company_role_id } = req.body;

  try {
    const newUserCompany = await prisma.usercompany.create({
      data: {
        company_id,
        user_id,
        company_role_id,
      },
    });
    res.status(201).json(newUserCompany);
  } catch (error) {
    res
      .status(500)
      .json({ message: "Error creating user-company association", error });
  }
};

// Update a UserCompany association by ID
export const updateUserFromCompany = async (req: Request, res: Response) => {
  const { id } = req.params;
  const { company_id, user_id, company_role_id } = req.body;

  try {
    const updatedUserCompany = await prisma.usercompany.update({
      where: { user_company_id: parseInt(id) },
      data: {
        company_id,
        user_id,
        company_role_id,
      },
    });

    res.status(200).json(updatedUserCompany);
  } catch (error) {
    res
      .status(500)
      .json({ message: "Error updating user-company association", error });
  }
};

// Delete a UserCompany association by ID
export const removeUserFromACompany = async (req: Request, res: Response) => {
  const { id } = req.params;

  try {
    await prisma.usercompany.delete({
      where: { user_company_id: parseInt(id) },
    });

    res.status(204).send();
  } catch (error) {
    res
      .status(500)
      .json({ message: "Error deleting user-company association", error });
  }
};

// create a user and assgin a company with role

export const createUserAndAssginCompany = async (
  req: Request,
  res: Response
) => {
  const { fname, lname, mname, company_id, email, password, company_role_id } =
    req.body;

  try {
    // Check if email already exists
    const existingUser = await prisma.users.findUnique({
      where: { email },
    });

    if (existingUser) {
      return res.status(400).json({ message: "Email already exists" });
    }

    // Hash the password
    const hashedPassword = await bcrypt.hash(password, 10);

    // Create the new user
    const newUser = await prisma.users.create({
      data: {
        email,
        password_hash: hashedPassword,
        first_name: fname,
        last_name: lname,
        middle_name: mname,
      },
    });

    // assign new user to the designated company

    const company = await prisma.usercompany.create({
      data: {
        user_id: newUser.user_id,
        company_id: company_id,
        company_role_id: company_role_id,
      },
    });

    // now assgin this user as a employee to the role table

    const user_roles = await prisma.user_roles.create({
      data: {
        user_id: newUser.user_id,
        role_id: parseInt(company_role_id),
      },
    });

    res.status(201).json({ message: "User Created & Assigned to the Company" });
  } catch (error) {
    res.status(500).json({ message: "Something Went Wrong", error });
  }
};

// Create a new company address
export const createCompanyAddress = async (req: Request, res: Response) => {
  const parsed = CompanySchema.CompanyAddressBase.extend({
    company_id: z.number(),
  }).safeParse(req.body);
  if (!parsed.success) {
    console.error(
      "POST > Create Company Address > Failed to parse request body\n",
      parsed.error
    );
    return res.status(400).json(parsed.error);
  }

  try {
    const newAddress = await prisma.company_address.create({
      data: parsed.data,
    });
    res.status(201).json({ message: "Data Added", data: newAddress });
  } catch (error) {
    res.status(500).json({ message: "Error creating company address", error });
  }
};

// Get company admin users
export const getCompanyAdminUsers = async (req: Request, res: Response) => {
  const { company_id } = req.params;
  const cid = Number.parseInt(company_id);

  try {
    const admins = await prisma.usercompany.findMany({
      where: {
        AND: {
          company_id: cid,
          company_role_id: {
            lte: 3,
          },
        },
      },
      include: {
        users: true,
        companies: true,
        roles: true,
      },
    });

    if (!admins) {
      return res.status(404).json({ message: "Company admin data not found" });
    }

    res.status(200).json(admins);
  } catch (error) {
    res
      .status(500)
      .json({ message: "Error retrieving company admin data", error });
  }
};

// Get a company address by ID
export const getCompanyAddressById = async (req: Request, res: Response) => {
  const { address_id } = req.params;

  try {
    const address = await prisma.company_address.findUnique({
      where: { address_id: parseInt(address_id) },
    });

    if (!address) {
      return res.status(404).json({ message: "Company address not found" });
    }

    res.status(200).json(address);
  } catch (error) {
    res
      .status(500)
      .json({ message: "Error retrieving company address", error });
  }
};

// Update a company address
export const updateCompanyAddress = async (req: Request, res: Response) => {
  const { address_id } = req.params;
  const {
    address_line_1,
    address_line_2,
    address_line_3,
    city_county,
    postcode,
    country,
  } = req.body;

  try {
    const updatedAddress = await prisma.company_address.update({
      where: { address_id: parseInt(address_id) },
      data: {
        postcode,
        address_line_1,
        address_line_2,
        address_line_3,
        city_county,
        country,
      },
    });

    res.status(200).json(updatedAddress);
  } catch (error) {
    res.status(500).json({ message: "Error updating company address", error });
  }
};

// Delete a company address
export const deleteCompanyAddress = async (req: Request, res: Response) => {
  const { address_id } = req.params;

  try {
    await prisma.company_address.delete({
      where: { address_id: parseInt(address_id) },
    });

    res.status(200).json({ message: "Company Address Deleted" });
  } catch (error) {
    res.status(500).json({ message: "Error deleting company address", error });
  }
};

// get all user of a company
export const getAllUserOfCompany = async (req: Request, res: Response) => {
  const { id } = req.params;
  const { page = 1, limit = 25, migrants_only = "false" } = req.query;

  const pageNum = parseInt(page as string);
  const limitNum = parseInt(limit as string);

  const skip = (pageNum - 1) * limitNum;

  try {
    var filters: any = {
      company_id: parseInt(id),
    };
    if (migrants_only === "true" || migrants_only === "1") {
      filters = {
        AND: {
          company_id: parseInt(id),
          users: {
            employee_data: {
              is_foreign: true,
            },
          },
        },
      };
    }
    var query = prisma.usercompany.findMany({
      where: filters,
      include: {
        users: {
          include: {
            employee_data: {
              include: {
                emp_type: true,
              },
            },
          },
        },
        roles: true,
        companies: false,
      },
      take: limitNum,
      skip: skip,
    });

    const users = await query;

    const total_rows = await prisma.usercompany.count({
      where: filters,
    });
    const total_page = Math.ceil(total_rows / limitNum);
    res.status(200).json({
      total_page,
      data_count: users.length,
      next_page: pageNum < total_page ? pageNum + 1 : undefined,
      previous_page: pageNum > 1 ? pageNum - 1 : undefined,
      data: users,
    });
  } catch (error) {
    console.error(
      "GET > Get all employees' data > Failed to get data >",
      error
    );
    res.status(500).json({ message: "Error Fetching Data", error });
  }
};

// Create a new company authorised detail
export const createCompanyAuthorisedDetail = async (
  req: Request,
  res: Response
) => {
  const {
    company_id,
    fname,
    lname,
    designation,
    phone_no,
    email,
    doc_link,
    offence_history,
  } = req.body;

  try {
    const newDetail = await prisma.company_authorised_details.create({
      data: {
        company_id,
        fname,
        lname,
        designation,
        phone_no,
        email,
        doc_link,
        offence_history,
      },
    });
    res.status(201).json(newDetail);
  } catch (error) {
    res
      .status(500)
      .json({ message: "Error creating company authorised detail", error });
  }
};

// Get a company authorised detail by ID
export const getCompanyAuthorisedDetailById = async (
  req: Request,
  res: Response
) => {
  const { authorised_id } = req.params;

  try {
    const detail = await prisma.company_authorised_details.findUnique({
      where: { authorised_id: parseInt(authorised_id) },
    });

    if (!detail) {
      return res
        .status(404)
        .json({ message: "Company authorised detail not found" });
    }

    res.status(200).json(detail);
  } catch (error) {
    res
      .status(500)
      .json({ message: "Error retrieving company authorised detail", error });
  }
};

// Update a company authorised detail
export const updateCompanyAuthorisedDetail = async (
  req: Request,
  res: Response
) => {
  const { authorised_id } = req.params;
  const {
    fname,
    lname,
    designation,
    phone_no,
    email,
    doc_link,
    offence_history,
    is_same_as_key_contact,
  } = req.body;

  try {
    const updatedDetail = await prisma.company_authorised_details.update({
      where: { authorised_id: parseInt(authorised_id) },
      data: {
        fname,
        lname,
        designation,
        phone_no,
        email,
        doc_link,
        offence_history,
      },
    });

    var companyID = updatedDetail.company_id;

    if (is_same_as_key_contact == true) {
      const updatedDetail = await prisma.company_l1_user.update({
        where: { company_id: companyID },
        data: {
          fname,
          lname,
          designation,
          phone_no,
          email,
          doc_link,
          offence_history,
        },
      });

      const keyContantDetails = await prisma.company_key_contact.update({
        where: { company_id: companyID },
        data: {
          fname,
          lname,
          designation,
          phone_no,
          email,
          doc_link,
          offence_history,
        },
      });
    }

    res.status(200).json(updatedDetail);
  } catch (error) {
    res
      .status(500)
      .json({ message: "Error updating company authorised detail", error });
  }
};

// Delete a company authorised detail
export const deleteCompanyAuthorisedDetail = async (
  req: Request,
  res: Response
) => {
  const { authorised_id } = req.params;

  try {
    await prisma.company_authorised_details.delete({
      where: { authorised_id: parseInt(authorised_id) },
    });

    res.status(204).send();
  } catch (error) {
    res
      .status(500)
      .json({ message: "Error deleting company authorised detail", error });
  }
};

// Create a new company L1 user
export const createCompanyL1User = async (req: Request, res: Response) => {
  const {
    company_id,
    fname,
    lname,
    designation,
    phone_no,
    email,
    doc_link,
    offence_history,
  } = req.body;

  try {
    const newL1User = await prisma.company_l1_user.create({
      data: {
        company_id,
        fname,
        lname,
        designation,
        phone_no,
        email,
        doc_link,
        offence_history,
      },
    });
    res.status(201).json(newL1User);
  } catch (error) {
    res.status(500).json({ message: "Error creating company L1 user", error });
  }
};

// Get a company L1 user by ID
export const getCompanyL1UserById = async (req: Request, res: Response) => {
  const { l1_user_id } = req.params;

  try {
    const l1User = await prisma.company_l1_user.findUnique({
      where: { l1_user_id: parseInt(l1_user_id) },
    });

    if (!l1User) {
      return res.status(404).json({ message: "Company L1 user not found" });
    }

    res.status(200).json(l1User);
  } catch (error) {
    res
      .status(500)
      .json({ message: "Error retrieving company L1 user", error });
  }
};

// Update a company L1 user
export const updateCompanyL1User = async (req: Request, res: Response) => {
  const { l1_user_id } = req.params;
  const {
    fname,
    lname,
    designation,
    phone_no,
    email,
    doc_link,
    offence_history,
    is_same_as_key_contact,
  } = req.body;

  try {
    const updatedL1User = await prisma.company_l1_user.update({
      where: { l1_user_id: parseInt(l1_user_id) },
      data: {
        fname,
        lname,
        designation,
        phone_no,
        email,
        doc_link,
        offence_history,
      },
    });

    var companyID = updatedL1User.company_id;

    if (is_same_as_key_contact == true) {
      const updatedDetail = await prisma.company_authorised_details.update({
        where: { company_id: companyID },
        data: {
          fname,
          lname,
          designation,
          phone_no,
          email,
          doc_link,
          offence_history,
        },
      });

      const keyContantDetails = await prisma.company_key_contact.update({
        where: { company_id: companyID },
        data: {
          fname,
          lname,
          designation,
          phone_no,
          email,
          doc_link,
          offence_history,
        },
      });
    }
    res.status(200).json(updatedL1User);
  } catch (error) {
    res.status(500).json({ message: "Error updating company L1 user", error });
  }
};

// Delete a company L1 user
export const deleteCompanyL1User = async (req: Request, res: Response) => {
  const { l1_user_id } = req.params;

  try {
    await prisma.company_l1_user.delete({
      where: { l1_user_id: parseInt(l1_user_id) },
    });

    res.status(204).send();
  } catch (error) {
    res.status(500).json({ message: "Error deleting company L1 user", error });
  }
};

// Create a new company trade detail
export const createCompanyTradeDetail = async (req: Request, res: Response) => {
  try {
    const {
      company_id,
      company_reg,
      type_of_company,
      trade_name,
      org_email,
      sector,
      change_of_name_5,
      faced_penaly_org,
    } = req.body;

    const newCompanyTradeDetail = await prisma.company_trade_details.create({
      data: {
        company_id,
        company_reg,
        type_of_company,
        trade_name,
        org_email,
        sector,
        change_of_name_5,
        faced_penaly_org,
      },
    });

    res.status(201).json(newCompanyTradeDetail);
  } catch (error) {
    console.error(
      "POST > Create Company Trade Details > Failed to create company trade details",
      error
    );
    res
      .status(500)
      .json({ error: "Error creating company trade detail", message: error });
  }
};

// Get all company trade details
export const getAllCompanyTradeDetails = async (
  req: Request,
  res: Response
) => {
  try {
    const companyTradeDetails = await prisma.company_trade_details.findMany();
    res.status(200).json(companyTradeDetails);
  } catch (error) {
    res
      .status(500)
      .json({ error: "Error fetching company trade details", message: error });
  }
};

// Get a single company trade detail by ID
export const getCompanyTradeDetailById = async (
  req: Request,
  res: Response
) => {
  try {
    const { id } = req.params;
    const companyTradeDetail = await prisma.company_trade_details.findUnique({
      where: { id: parseInt(id) },
    });

    if (!companyTradeDetail) {
      return res.status(404).json({ error: "Company trade detail not found" });
    }

    res.status(200).json(companyTradeDetail);
  } catch (error) {
    res
      .status(500)
      .json({ error: "Error fetching company trade detail", message: error });
  }
};

// Update a company trade detail
export const updateCompanyTradeDetail = async (req: Request, res: Response) => {
  try {
    const { id } = req.params;
    const {
      company_id,
      company_reg,
      type_of_company,
      trade_name,
      org_email,
      sector,
      change_of_name_5,
      faced_penaly_org,
    } = req.body;

    const updatedCompanyTradeDetail = await prisma.company_trade_details.update(
      {
        where: { id: parseInt(id) },
        data: {
          company_id,
          company_reg,
          type_of_company,
          trade_name,
          org_email,
          sector,
          change_of_name_5,
          faced_penaly_org,
        },
      }
    );

    res.status(200).json(updatedCompanyTradeDetail);
  } catch (error) {
    console.error(
      "PUT > Update Company Trade Details > Failed to update company trade details",
      error
    );
    res
      .status(500)
      .json({ error: "Error updating company trade detail", message: error });
  }
};

// Delete a company trade detail
export const deleteCompanyTradeDetail = async (req: Request, res: Response) => {
  try {
    const { id } = req.params;

    await prisma.company_trade_details.delete({
      where: { id: parseInt(id) },
    });

    res.status(204).send();
  } catch (error) {
    res
      .status(500)
      .json({ error: "Error deleting company trade detail", message: error });
  }
};

// Create a new company trading hour
export const createSingleCompanyTradingHour = async (
  req: Request,
  res: Response
) => {
  try {
    const parsed = CompanySchema.CompanyTradingHourBase.safeParse(req.body);

    if (!parsed.success) {
      console.error(
        "POST > Create Single Company Trading Hour > Failed to parse request body\n",
        parsed.error
      );
      return res.status(400).json(parsed.error);
    }

    console.log(
      "POST > Create Single Company Trading Hour > Request Body >",
      parsed.data
    );

    const newCompanyTradingHour = await prisma.company_trading_hour.create({
      data: parsed.data,
    });

    res.status(201).json(newCompanyTradingHour);
  } catch (error) {
    console.error(
      "POST > Faile to Create Single Company Trading Hour >",
      error
    );
    res
      .status(500)
      .json({ error: "Error creating company trading hour", message: error });
  }
};

// Create many new company trading hours
export const createCompanyTradingHour = async (req: Request, res: Response) => {
  try {
    // Expected a list of data, data for 7 days of the week
    const parsed = z
      .array(CompanySchema.CompanyTradingHourBase)
      .safeParse(req.body);
    if (!parsed.success) {
      console.error(
        "POST > Create Company Trading Hours > Failed to parse request body\n",
        parsed.error
      );
      return res.status(400).json(parsed.error);
    }

    console.log("POST > Create Company Trading Hours >", parsed.data);

    const newCompanyTradingHour = await prisma.company_trading_hour.createMany({
      data: parsed.data,
      skipDuplicates: true,
    });

    res.status(201).json({
      message: `Insertion(s) successful: ${newCompanyTradingHour.count}.`,
    });
  } catch (error) {
    console.error(
      "POST > Create Company Trading Hours > Failed to Create company trading hours >",
      error
    );
    res
      .status(500)
      .json({ error: "Error creating company trading hour", message: error });
  }
};

// Get all company trading hours
export const getAllCompanyTradingHours = async (
  req: Request,
  res: Response
) => {
  try {
    const { cid } = req.params;
    const companyTradingHours = await prisma.company_trading_hour.findMany({
      where: { company_id: parseInt(cid) },
    });
    res.status(200).json(companyTradingHours);
  } catch (error) {
    res
      .status(500)
      .json({ error: "Error fetching company trading hours", message: error });
  }
};

// Get a single company trading hour by ID
export const getCompanyTradingHourById = async (
  req: Request,
  res: Response
) => {
  try {
    const { id } = req.params;
    const companyTradingHour = await prisma.company_trading_hour.findUnique({
      where: { id: parseInt(id) },
    });

    if (!companyTradingHour) {
      return res.status(404).json({ error: "Company trading hour not found" });
    }

    res.status(200).json(companyTradingHour);
  } catch (error) {
    res
      .status(500)
      .json({ error: "Error fetching company trading hour", message: error });
  }
};

// Update a company trading hour
export const updateCompanyTradingHour = async (req: Request, res: Response) => {
  try {
    const { id } = req.params;
    const { company_id, day_name, trade_status, opening_time, closing_time } =
      req.body;

    const updatedCompanyTradingHour = await prisma.company_trading_hour.update({
      where: { id: parseInt(id) },
      data: {
        company_id,
        day_name,
        trade_status,
        opening_time,
        closing_time,
      },
    });

    res.status(200).json(updatedCompanyTradingHour);
  } catch (error) {
    res
      .status(500)
      .json({ error: "Error updating company trading hour", message: error });
  }
};

// Update company trading hours by company id
export const updateCompanyTradingHourBulk = async (
  req: Request,
  res: Response
) => {
  try {
    const parsed = z
      .array(CompanySchema.CompanyTradingHourBase)
      .safeParse(req.body);

    if (!parsed.success) {
      console.error(
        "PUT > Update Company Trading Hours in Bulk > Failed to parse request body\n",
        parsed.error
      );
      return res.status(400).json(parsed.error);
    }

    // Using raw sql for better performance, transaction is linear and slow.
    const companyIds = parsed.data.map((th) => th.company_id);
    const dayNames = parsed.data.map((th) => th.day_name);
    const tradeStatusCases = parsed.data
      .map(
        (th) =>
          `WHEN company_id = ${th.company_id} AND day_name = '${th.day_name}' THEN ${th.trade_status}`
      )
      .join(" ");
    const openingTimeCases = parsed.data
      .map(
        (th) =>
          `WHEN company_id = ${th.company_id} AND day_name = '${th.day_name}' THEN '${th.opening_time}'`
      )
      .join(" ");
    const closingTimeCases = parsed.data
      .map(
        (th) =>
          `WHEN company_id = ${th.company_id} AND day_name = '${th.day_name}' THEN '${th.closing_time}'`
      )
      .join(" ");

    const sql = `
  UPDATE company_trading_hour
  SET
    trade_status = CASE ${tradeStatusCases} ELSE trade_status END,
    opening_time = CASE ${openingTimeCases} ELSE opening_time END,
    closing_time = CASE ${closingTimeCases} ELSE closing_time END,
    updated_at = NOW()
  WHERE company_id IN (${companyIds.join(", ")})
    AND day_name IN (${dayNames.map((name) => `'${name}'`).join(", ")});
`;

    const result = await prisma.$executeRawUnsafe(sql);

    res.status(200).json({
      message: `Updated company trading hours. Affected rows: ${result}`,
    });
  } catch (error) {
    console.error(
      "PUT > Failed to Update Company Trading Hours In Bulk >",
      error
    );
    res
      .status(500)
      .json({ error: "Error updating company trading hours", message: error });
  }
};

// Delete a company trading hour
export const deleteCompanyTradingHour = async (req: Request, res: Response) => {
  try {
    const { id } = req.params;

    await prisma.company_trading_hour.delete({
      where: { id: parseInt(id) },
    });

    res.status(204).send();
  } catch (error) {
    res
      .status(500)
      .json({ error: "Error deleting company trading hour", message: error });
  }
};

// Create a new company key contact
export const createCompanyKeyContact = async (req: Request, res: Response) => {
  try {
    const {
      company_id,
      fname,
      lname,
      designation,
      phone_no,
      email,
      doc_link,
      offence_history,
    } = req.body;

    const newCompanyKeyContact = await prisma.company_key_contact.create({
      data: {
        company_id,
        fname,
        lname,
        designation,
        phone_no,
        email,
        doc_link,
        offence_history,
      },
    });

    res.status(201).json(newCompanyKeyContact);
  } catch (error) {
    res
      .status(500)
      .json({ error: "Error creating company key contact", message: error });
  }
};

// Get all company key contacts
export const getAllCompanyKeyContacts = async (req: Request, res: Response) => {
  try {
    const companyKeyContacts = await prisma.company_key_contact.findMany();
    res.status(200).json(companyKeyContacts);
  } catch (error) {
    res
      .status(500)
      .json({ error: "Error fetching company key contacts", message: error });
  }
};

// Get a single company key contact by ID
export const getCompanyKeyContactById = async (req: Request, res: Response) => {
  try {
    const { key_contact_id } = req.params;
    const companyKeyContact = await prisma.company_key_contact.findUnique({
      where: { key_contact_id: parseInt(key_contact_id) },
    });

    if (!companyKeyContact) {
      return res.status(404).json({ error: "Company key contact not found" });
    }

    res.status(200).json(companyKeyContact);
  } catch (error) {
    res
      .status(500)
      .json({ error: "Error fetching company key contact", message: error });
  }
};

// Update a company key contact
export const updateCompanyKeyContact = async (req: Request, res: Response) => {
  try {
    const { key_contact_id } = req.params;
    const {
      company_id,
      fname,
      lname,
      designation,
      phone_no,
      email,
      doc_link,
      offence_history,
      is_same_as_key_contact,
    } = req.body;

    const updatedCompanyKeyContact = await prisma.company_key_contact.update({
      where: { key_contact_id: parseInt(key_contact_id) },
      data: {
        company_id,
        fname,
        lname,
        designation,
        phone_no,
        email,
        doc_link,
        offence_history,
      },
    });

    var companyID = updatedCompanyKeyContact.company_id;

    if (is_same_as_key_contact == true) {
      const updatedDetail = await prisma.company_l1_user.update({
        where: { company_id: companyID },
        data: {
          fname,
          lname,
          designation,
          phone_no,
          email,
          doc_link,
          offence_history,
        },
      });

      const keyContantDetails = await prisma.company_authorised_details.update({
        where: { company_id: companyID },
        data: {
          fname,
          lname,
          designation,
          phone_no,
          email,
          doc_link,
          offence_history,
        },
      });
    }

    res.status(200).json(updatedCompanyKeyContact);
  } catch (error) {
    res
      .status(500)
      .json({ error: "Error updating company key contact", message: error });
  }
};

// Delete a company key contact
export const deleteCompanyKeyContact = async (req: Request, res: Response) => {
  try {
    const { key_contact_id } = req.params;

    await prisma.company_key_contact.delete({
      where: { key_contact_id: parseInt(key_contact_id) },
    });

    res.status(204).send();
  } catch (error) {
    res
      .status(500)
      .json({ error: "Error deleting company key contact", message: error });
  }
};

//Get All Detalis of the company via company ID

export const getCompanyDetaisByCompanyID = async (
  req: Request,
  res: Response
) => {
  try {
    const { companyID } = req.params;

    const company = await prisma.companies.findUnique({
      where: { company_id: parseInt(companyID) },
      include: {
        company_address: true,
        company_authorised_details: true,
        company_docs_db: {
          where: {
            is_active: true,
          },
        },
        company_key_contact: true,
        company_l1_user: true,
        company_trade_details: true,
        company_trading_hour: true,
        leave_approvers: false,
        usercompany: false,
      },
    });

    if (!company) {
      return res.status(404).json({ error: "Company  not found" });
    }

    res.status(200).json(company);
  } catch (error) {
    console.error(
      "GET > Get company by id > Failed to get company by id\n",
      error
    );
    res.status(500).json({ error: "Error fetching", message: error });
  }
};

//search for company

export const searchCompany = async (req: Request, res: Response) => {
  const { name } = req.query;

  try {
    const result = await prisma.companies.findMany({
      where: {
        company_name: {
          contains: name as string,
        },
      },
    });

    res.status(200).json(result);
  } catch (error) {
    console.error(
      "GET > Get company by id > Failed to get company details\n",
      error
    );
    res.status(500).json({ message: "Error fetching company", error });
  }
};

export async function getCompanyShowcase(
  req: Request,
  res: Response,
  next: NextFunction
) {
  try {
    // Take latest 15 companies for showcase,
    // Normally it would've been selected renowned companies,
    // But that's for the stakeholders to decide
    const companies = await prisma.companies.findMany({
      select: {
        company_name: true,
        logo: true,
      },
      take: 15,
      orderBy: {
        company_id: "desc",
      },
    });

    return res.status(200).json(companies);
  } catch (err) {
    console.error(
      "GET > Get Company Showcase > Failed to get company showcase",
      err
    );
    next(err);
  }
}

export async function getCompanyCount(
  req: Request,
  res: Response,
  next: NextFunction
) {
  try {
    const total = await prisma.companies.count();
    const snapped = Math.floor(total / 10) * 10;
    var ans = snapped < 1 ? total : snapped;

    return res.status(200).json({ ans });
  } catch (err) {
    console.error(
      "GET > Get company count snapped down to nearest 10s > Failed to get total count",
      err
    );
    next(err);
  }
}

//#region Update User

const userUpdateSchema = z.object({
  first_name: z.string().min(1),
  last_name: z.string().min(1),
  middle_name: z.string().min(0),
  email: z.string().email(),
});

export const updateUser = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { user_id } = req.params;
  const userId = Number.parseInt(user_id);

  try {
    const { error, data } = userUpdateSchema.safeParse(req.body);

    if (error) {
      console.error(
        "PATCH > Failed to update user info > Invalid Request Body",
        error
      );
      return res.status(400).json({
        message: "Invalid request body",
        error,
      });
    }

    await prisma.users.update({
      where: { user_id: userId },
      data: {
        ...data,
      },
    });
  } catch (error) {
    console.error("PATCH > Failed to update user info", error);
    next(error);
  }
};

//#endregion
